C DILATOMETRIC ANALYSIS PROGRAM 4 September 1989
C
C H. K. D. H. Bhadeshia.  Oak Ridge Book, p.17
C
C Details published in
C  H. K. D. H. Bhadeshia, S. A. David, J. M. Vitek and R. W. Reed
C  Materials Science and Technology 7 (1991) 686--698
C
C To calculate the volume fraction of transformation
C   from austenite to ferrite, from a measure of the 
C   length change (measured using a dilatometer). 
C Austenite and ferrite are allotropes of iron, with 
C   a face--centered and body--centered cubic crystal 
C   structure respectively. There is a change in density
C   when austenite transforms to ferrite. This causes a
C   dimensional change, which can be monitored to follow
C   the transformation as it happens. The instrument used
C   to monitor dimensional change (length change) is called
C   a dilatometer. 
C
C  Typical Dataset
C
C  1 (No. of L-T pairs) C Si Mn Ni Mo Cr V wt.%
C  0.03 (Amount of C trapped in ferrite, wt.%) 
C  EXA EXG (expansivities of ferrite and austenite respectively)
C  0.002 (relative length change) 415.0 (centigrade)
C   
C    End of typical dataset
C
C -------------------------------------------------------------------
C
C Lattice parameter of austenite calculated from Dyson,
C taking account of all alloying additions. 
C D J Dyson and B Holmes, Journal of the Iron and Steel Institute,
C  277 (1970) 469.
C
C
C If problems with Newton iteration, adjust SENS to lower value 
C
C Variation of austenite lattice parameter as a function of carbon
C 3.573+.033*XBAR  
C
C XML   carbon content of residual austenite, mole fraction
C XWT   carbon content of residual austenite, weight percent
C AA    ferrite lattice parameter at 298K
C GG    austenite lattice parameter at 298K
C AAT   ferrite lattice parameter at temperature
C AGT   lattice parameter for austenite with mean C conc. (XBAR) at temperature 
C AGTX  carbon enriched austenite lattice parameter at temperature
C VA    is the volume fraction of ferrite
C T     is the temperature in Centigrade
C EXA   expansion coefficient of ferrite, 1/K
C EXG   expansion coefficient of austenite, 1/K
C
C ----------------------------------------------------------------
C
      IMPLICIT NONE   
      DOUBLE PRECISION XBAR,SI,MN,NI,MO,CR,VAN,XALPHA,EXA,EXG
      DOUBLE PRECISION RTEMP,PP,SENS,GG,AA,AAT,AGT,VA,AGTX,XG,A1
      DOUBLE PRECISION AV,VA1,VA2,A11,A12,FDASH,XGM,PFE
      DOUBLE PRECISION RMSI,RMMN,RMNI,RMMO,RMCR,RMVAN,RMFE
      DOUBLE PRECISION L(100),T(100)
      INTEGER IDATA,III,I,IALPH
C
C Physical constants used in calculations.
C Reference : "Fundamental physical constants and energy conversion factors",
C National Physical Laboratory.
C
C
C  AV is the Avogadro constant (in 1/(mole)
C
      AV=6.0221367D+23
C
C set value of room temperature in centigrade
C
      RTEMP=25.0D+00
C
C Set the value for the pure iron lattice parameter at room temperature
C
      PFE = 2.8664D-10
C
      SENS=0.8D+00
C
C OPEN INPUT AND OUTPUT FILES:
C          INPUT FILE IS CALLED LENGTH.DAT
C          OUTPUT FILE IS CALLED OUT.DAT
C
      OPEN(UNIT=5,STATUS='OLD',FILE='LENGTH.DAT')
      OPEN(UNIT=6,FILE='OUT.DAT')
C       
      READ(5,*)IDATA,XBAR,SI,MN,NI,MO,CR,VAN
      READ(5,*)XALPHA,EXA,EXG
C
      CALL MAP_STEEL_FERR(XBAR,SI,MN,NI,MO,CR,VAN,XALPHA,
     & PFE,PP)
      WRITE(6,1) XALPHA, EXA,EXG,RTEMP,PP
C
      WRITE (6,2) XBAR,MN,SI,NI,CR,MO,VAN
C
      DO 100 I=1,IDATA
         READ (5,*) L(I),T(I)
  100 CONTINUE
C
      GG = 0.00095*MN-0.0002*NI+0.0006*CR+0.0031*MO+0.0018*VAN
     &   +3.573+0.033*XBAR
C
      AA = 2.870D+00
C
      DO 160 IALPH=1,5
         AA=AA+0.001D+00
         WRITE(6,3) AA
         DO 150 I=1,IDATA
            III = 0
            AAT = AA*(1.0D+00 + (T(I)-RTEMP)*EXA)
            AGT = GG*(1.0D+00 + (T(I)-RTEMP)*EXG)
            VA  = 0.8D+00
C   
  110       CALL MAP_STEEL_PARAM(AGTX,AGT,AAT,VA,XBAR,XALPHA,XG,
     &                     EXG,T(I),AV,RTEMP)
            CALL MAP_STEEL_FPARAM(AGTX,AGT,AAT,VA,L(I),A1)
            IF(DABS(A1) .GT. 0.1D-04) GOTO 120
            GOTO 140
C
C Newton iteration procedure
C
  120       VA1=VA+1.0D-11
            VA2=VA-1.0D-11
            CALL MAP_STEEL_FPARAM(AGTX,AGT,AAT,VA1,L(I),A11)
            CALL MAP_STEEL_FPARAM(AGTX,AGT,AAT,VA2,L(I),A12)
            FDASH=(A11-A12)/(VA1-VA2)
            III=III+1
            IF(III .GT. 11135) THEN
               WRITE(*,4)
               STOP
            ENDIF
C
C FDASH is the differential of function A1
C
            VA = VA - SENS*(A1/FDASH)
  130       IF (VA .GT. 0.99) THEN
               VA = VA*0.8D+00
               GOTO 130
            ENDIF
            GOTO 110
C
C end of iteration when jump to 140
C 
  140       CALL MAP_STEEL_ATM(XG,XGM,XBAR,SI,MN,NI,MO,CR,VAN,
     &                      RMSI,RMMN,RMNI,RMMO,RMCR,RMVAN,RMFE)
            WRITE(6,5) T(I),L(I),VA,XG,XGM,AAT,AGT,AGTX
  150    CONTINUE
  160 CONTINUE
      STOP
    1 FORMAT(' Amount of carbon trapped in ferrite, wt.% =',F8.4/
     &       '          Linear expansivity of alpha, 1/K =',D12.5/
     &       '          Linear expansivity of gamma, 1/K =',D12.5/
     &       '          Ambient temperature, Centigrade  =',F8.0/
     &       '          Calculated ferrite parameter, m  =',D12.4)
    2 FORMAT (/' STEEL COMPOSITION, wt.% '/
     &    7H Carbon,F10.3/10H Manganese,F7.3,10X,8H Silicon,F9.3,
     & 10X,7H Nickel,F10.3/9H Chromium,F8.3,10X,11H Molybdenum F6.3,
     & 10X,9H Vanadium,F8.3//
     & ' Ctemp Del-L    V-alpha   Xg-wt%    Xg-Mfrc ',
     & ' AAT Ang   AGT Ang  AGTX Ang   AA'/)
    3 FORMAT(72X,F8.5)
    4 FORMAT(' *****************************'///
     &       '  TOO MANY ITERATION STEPS, adjust SENS'///
     &       ' ******************************************')
    5 FORMAT(F6.0,F8.5,F8.3,2F10.4,3F10.5)
      END
C
C**************************************************************
C Subroutine MAP_STEEL_FERR
C
C
C Calculate ferrite lattice parameter P at room temperature for alloy steel
C Data from Leslie, concentrations input as wt.%, balance iron
C Mo effect F. Wever, Zeitschrift fur Metallkunde 20 (1928) 363-370
C CALP is wt.% carbon dissolved in ferrite
C Vanadium coefficient from Hume-Rothery an Sutton (Structure of Iron+Alloys)
C Set PFE as the parameter of pure iron at room temperature
C Value of 2.8664 Ansgtroms for PFE from National Bureau of Standards (USA)
C Circ. 539, 4 3 (1955)
C H. K. D. H. Bhadeshia, 6 September 1989
C
      SUBROUTINE MAP_STEEL_FERR(C,SI,MN,NI,MO,CR,V,CALP,PFE,P)
      IMPLICIT NONE 
      DOUBLE PRECISION C,SI,MN,NI,MO,CR,V,CALP,P,PFE,ASI,AMN,
     &ANI,ACR,AMO,AV,FE,AFE,AC,M,MSI,MMN,MNI,MCR,MMO,MFE,MC,
     &MV,V1,V2,CC
C
C
        ASI=SI/28.0855
        AMN=MN/54.938
        ANI=NI/58.71
        ACR=CR/51.996
        AMO=MO/95.94
        AV=V/50.9415
C        AW=W/183.85
C        ACO=CO/58.9332
C
      FE=100.0-C-SI-MN-NI-MO-CR-V
      AFE=FE/55.847
      AC=C/12.011
C
      M=AC+ASI+AMN+ANI+AMO+ACR+AV+AFE
        MSI=ASI/M
        MMN=AMN/M
        MNI=ANI/M
        MCR=ACR/M
        MMO=AMO/M
C        MW=AW/M
C        MCO=ACO/M
        MFE=AFE/M
        MC=AC/M
        MV=AV/M
C
C expansion caused by carbon 
C Reference: C. S. Roberts, Trans. AIME, 197 (1953) 203
C
            V1=PFE*PFE*PFE
            V2=(PFE - 0.013D-10*CALP)*(PFE - 0.013D-10*CALP)*
     &      (PFE + 0.116D-10*CALP)
            CC=(V2-V1)/(3.0*PFE*PFE)
C
      P=PFE+CC - MSI*3.0D-12 + MMN*0.06D-10 + MNI*0.07D-10
     &+ MMO*0.31D-10 + MCR*0.05D-10 + MV*0.096D-10 
C
        RETURN
        END
C
C**************************************************************
      SUBROUTINE MAP_STEEL_FPARAM(AGTX,AGT,AAT,VA,DL,A1)
C
      IMPLICIT NONE                          
      DOUBLE PRECISION AGTX,AGT,AAT,VA,DL,A1,A,A2
C
      A  = VA/(1.0D0-VA)                   
      A2 = 1.0D0/(1.0D0 + (2.0D0/A)*(AAT**3/AGTX**3))
      A1 = 1.0D0-3.0D0*DL-((AGT**3)/
     &         ((AGTX**3) + (2.0D0*(AAT**3)-(AGTX**3))*A2))
      RETURN
      END
C
C**************************************************************
C Subroutine MAP_STEEL_PARAM
C
C Calculates the lattice parameter of enriched austenite, after a
C degree of transformation to ferrite.
C
      SUBROUTINE MAP_STEEL_PARAM(AGTX,AGT,AAT,VA,XBAR,XALPHA,XG,EXG,
     &T,AV,RTEMP)
C
      IMPLICIT NONE                            
      DOUBLE PRECISION AGTX,AGT,AAT,VA,XBAR,XALPHA,XG,EXG,T,AGES,
     &                 RHOA,RHOG,RTEMP,AV
      INTEGER I
C
C  Guess a value AGES of AGTX and then iterate   
C
      AGTX = AGT
      AGES = AGTX
      I    = 1    
 100  IF(I .EQ. 1) GOTO 110
      AGES = AGTX
C
C Make the approximation that density Rho depends
C   only on atomic wt. of iron
C   See p.58, Oak Ridge Book
C
C Calculate densities from the unit cell dimensions for ferrite and austenite
C
  110 RHOA = 55.847*2.0/(AV*AAT*AAT*AAT*1.0D-30)
      RHOG = 55.847*4.0/(AV*AGTX*AGTX*AGTX*1.0D-30)
      XG   = (VA*RHOA*(XBAR-XALPHA) + XBAR*RHOG*(1.0-VA))
     &                   /((1.0-VA)*RHOG)
C
C calculate new value of enriched austenite lattice parameter 
C
      AGTX = AGT + 0.033*(XG-XBAR)*(1.0D0 + (T-RTEMP)*EXG) 
      I    = 2 
      IF ( (AGTX/AGES) .GT. 1.0001D0 .OR. 
     &     (AGTX/AGES) .LT. 0.9999D0) GOTO 100
      RETURN
      END
C**************************************************************
C
C converts wt.% carbon in enriched austenite, to mole fraction,
C given the alloy composition prior to enrichment, and the wt.%
C carbon following enrichment.
C
      SUBROUTINE MAP_STEEL_ATM(XG,XGM,XBAR,DSI,DMN,DNI,DMO,DCR,
     & DVAN,MSI,MMN,MNI,MMO,MCR,MVAN,MFE)
      IMPLICIT NONE                           
      DOUBLE PRECISION XG,XGM,XBAR,DSI,DMN,DNI,DMO,DCR,DVAN,
     &MSI,MMN,MNI,MMO,MCR,MVAN,MFE,M,BSI,BMN,BNI,BCR,BMO,BVAN,
     &BFE,BC,C,SI,MN,NI,MO,CR,VAN,ASI,AMN,ANI,ACR,AMO,AVAN,
     &AFE,AC,Z,ZSI,ZMN,ZNI,ZMO,ZCR,ZVAN,ZFE,FE
      INTEGER I
C
        I=0
C
C atomic weights
C
        BSI=28.0855
        BMN=54.938
        BNI=58.71
        BCR=51.996
        BMO=95.94
        BVAN=50.94
        BFE=55.847
        BC=12.011
C
C
        C=XBAR
        SI=DSI
        MN=DMN
        NI=DNI
        MO=DMO
        CR=DCR
        VAN=DVAN
C
C Divide wt.% by atomic weights
C
 1       ASI=SI/BSI    
        AMN=MN/BMN
        ANI=NI/BNI
        ACR=CR/BCR
        AMO=MO/BMO
        AVAN=VAN/BVAN
        AFE=FE/BFE
        AC=C/BC
C
C Calculate susbtitutional atom fractions
C
      Z=ASI+AMN+ANI+ACR+AMO+AVAN+AFE
      M=Z+AC
        ZSI=ASI/Z
        ZMN=AMN/Z
        ZNI=ANI/Z
        ZCR=ACR/Z
        ZMO=AMO/Z
        ZVAN=AVAN/Z
        ZFE=AFE/Z
C
        IF(I .GT. 0) GOTO 3
C
C new values allowing for carbon enrichment
C
        SI=ZSI*BSI*(1.0D+02-XG)/BFE
        MN=ZMN*BMN*(1.0D+02-XG)/BFE
        NI=ZNI*BNI*(1.0D+02-XG)/BFE
        MO=ZMO*BMO*(1.0D+02-XG)/BFE
        CR=ZCR*BCR*(1.0D+02-XG)/BFE
        VAN=ZVAN*BVAN*(1.0D+02-XG)/BFE
        FE=ZFE*BFE*(1.0D+02-XG)/BFE
C
      I=2
      C=XG
      GOTO 1
C
C new values of mole fractions
C
3       MSI=ASI/M
        MMN=AMN/M
        MNI=ANI/M
        MCR=ACR/M
        MMO=AMO/M
        MVAN=AVAN/M
C       MCO=ACO/M
        MFE=AFE/M
      XGM=AC/M
C
        RETURN
        END
C
